# $Id: raterecordcookie.py 2969 2010-01-25 14:31:42Z anton $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-9.7.1/src/delfttilburg/raterecordcookie.py $
## begin license ##
#
#    "Delft-Tilburg" (delfttilburg) is a package containing shared code
#    between the Delft "Discover" and Tilburg "Beter Zoeken & Vinden" projects.
#    Both projects are based on Meresco Software (http://meresco.com)
#    Copyright (C) 2008-2009 Technische Universiteit Delft http://www.tudelft.nl
#    Copyright (C) 2008-2009 Universiteit van Tilburg http://www.uvt.nl
#    Copyright (C) 2008-2009 Seek You Too (CQ2) http://www.cq2.nl
#
#    This file is part of "Delft-Tilburg".
#
#    "Delft-Tilburg" is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    "Delft-Tilburg" is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with "Delft-Tilburg"; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
## end license ##
from __future__ import with_statement
from md5 import md5
from uuid import uuid4 as randomUuid
from os.path import exists
from pickle import load as loadPickleData, dump

MAX_AGE = 60*60*24*365*10 # 10 years

def store(filename, value):
    with open(filename, 'w') as f:
        dump(value, f)

def load(filename):
    if filename and exists(filename):
        with open(filename) as f:
            return loadPickleData(f)
    return {}

class RateRecordCookie(object):
    def __init__(self, aFilename = None):
        self._name = "recordListId"
        self._id2list = {}
        self._store = lambda: None
        if aFilename:
            self._id2list = load(aFilename)
            self._store = lambda: store(aFilename, self._id2list)

    def _createId(self):
        return str(randomUuid())

    def _hashRecordId(self, aRecordId):
        return md5(aRecordId).hexdigest()

    def recordInList(self, Headers, recordId):
        listId = self._getListIdentifier(Headers)
        if listId and listId in self._id2list:
            return self._hashRecordId(recordId) in self.getList(listId)
        return False

    def addRecordToList(self, Headers, recordId):
        result = None
        listId = self._getListIdentifier(Headers)
        if not listId:
            listId = self._createId()
            result = 'Set-Cookie: %s=%s; Path=/; Version=1; Max-Age=%s' % (self._name, listId, MAX_AGE)
        self.getList(listId).append(self._hashRecordId(recordId))
        self._store()
        return result

    def getList(self, identifier):
        if not identifier in self._id2list:
            self._id2list[identifier] = []
        return self._id2list[identifier]
    
    def _getListIdentifier(self, Headers):
        splittedCookies = (cookie.split('=', 1) for cookie in Headers.get('Cookie', '').split(';') if cookie.strip())
        strippedCookies = ((name.strip(), value.strip()) for name, value in splittedCookies)
        oldValue = [value for (name, value) in strippedCookies if name == self._name]
        if oldValue:
            return oldValue[0]
        return None
